### empirical data 

# set working directory
dir <- "/home/jgb/projects/aac"
setwd(dir)

# load some funcitons into memory
source("code/r/community_change_functions.r")
source("code/r/process_empirical_aac.r")
source("code/r/make_naac_vector.r")
source("code/r/perform_simulation.r")

# read in data
aac_data <- read.table("data/empirical.csv", row.names=1, header=FALSE, sep=",", )

# get acc vector
dataset <- "c1"
acc_data <- extract_acc_vector(acc_data,1) 

#########################
### simulation parameters

# initial community size
N = length(acc_data)

# size of species pool
P = 73

# number of time steps (5 My, steps of 1 Ky)
t_max = 5E3

# rate of extinction from community
pex <- 0.01
# rate of immigration to community
pim <- pex * N / (P - N) 

set.seed(67234456)
tr <- perform_simulation(P, N, pex, pim, t_max)
tr_nacc_vec <- make_naac_vector(tr, t_max)

# calculate empirical naac vectors
# with simulaiton t_max
data_nacc_vec <- make_naac_vector(acc_data, t_max)

# make a quick plot, check all is working
sum(abs(tr_nacc_vec - data_nacc_vec))
plot(tr_nacc_vec,xlim=c(3000,5000))
points(data_nacc_vec,col="blue")


###################################
### Infer pex 

set.seed(12984)
ns       <- 10000
sim_sum  <- list()
sim_tab  <- mat.or.vec(ns,2)
pex_pr   <- runif(ns, min=0.001, max=0.05)

for (j in 1:ns) {

   cat("up to sim:", j, "\n")
   trj <- perform_simulation(P, N, pex_pr[j], pex_pr[j] * N / (P - N), t_max)
   trj_nacc <- make_naac_vector(trj, t_max)

   sim_tab[j, 1] <- pex_pr[j]
   sim_tab[j, 2] <- sum(abs(trj_nacc - data_nacc_vec))
   sim_sum[[j]]  <- trj

}

pdf(file=paste("figures/", dataset, "_abc.pdf", sep=""))
plot(sim_tab[,1],sim_tab[,2], xlab="pex", ylab="summary statistic")
dev.off()

i_best <- which(sim_tab[,2] < 155)
pex_mean_best <- mean(sim_tab[i_best,1])

i_best_sim <- which.min(sim_tab[,2])
best_pex   <- sim_tab[i_best_sim,1]
best_sim_tr   <- sim_sum[[i_best_sim]]

best_naac <- make_naac_vector(best_sim_tr, t_max)
# quick check
sum(abs(best_naac - data_nacc_vec))

similar_pex_to_best <- which( abs(sim_tab[,1]-best_pex) < 0.00001)
#[1] 5583 7658
i_example_sim <- 5583
example_pex   <- sim_tab[i_example_sim,1]
example_sim_tr <- sim_sum[[i_example_sim]]
example_naac <- make_naac_vector(example_sim_tr, t_max)


pdf(file=paste("figures/", dataset, "_naac_plots.pdf", sep=""))
plot(best_naac,xlim=c(3000,5000), xlab="step", ylab="arrivals (normalized)")
points(data_nacc_vec,col="blue")
points(example_naac,col="gray")
dev.off()

png(file=paste("figures/FigS3D.png", sep=""), units='in', res=320, height=5, width=5)
plot(data_nacc_vec,xlim=c(3000,5000), xlab="time step", ylab="arrivals (normalized)", cex=1, col="blue")
points(best_naac,col="black", cex=0.5)
dev.off()


#####################################
### Two rate classes

pim  <- 0.0565
N_c2 <- 24 
P_c2 <- 25

P_c1 <- P - P_c2
N_c1 <- N - N_c2

pim2 <- pim
pex2 <- pim2 * (P_c2 - N_c2) / N_c2

pim2 <- pim
pex  <- pim * ( P_c1 - N_c1 ) / N_c1

N_c1_check <- pim / (pim + pex) * P_c1

inds <- P_c2

classes <- 2

set.seed(9823573)
rc2 <- perform_simulation(P, N, pex, pim, t_max, classes=classes, pex2=pex2, pim2=pim2,inds=inds)
rc2_nacc_vec <- make_naac_vector(rc2, t_max)

# make a quick plot, check all is working
sum(abs(rc2_nacc_vec - data_nacc_vec))

plot(rc2_nacc_vec,xlim=c(3000,5000))
points(data_nacc_vec,col="blue")


######################################
nsims <- 1000
set.seed(987347)
post_sim_tab <- mat.or.vec(nsims,2)
colnames(post_sim_tab) <- c("oneClass", "twoClass")
tc_naac_list <- list()
sc_naac_list <- list()

for (k in 1:nsims) {

   cat("up to sim:", k, "\n")
   scs     <- perform_simulation(P, N, pex_mean_best, pex_mean_best * N / (P - N), t_max)
   sc_nacc <- make_naac_vector(scs, t_max)
   post_sim_tab[k, 1] <- sum(abs(sc_nacc - data_nacc_vec))
   sc_naac_list[[k]] <- sc_nacc

   tcs     <- perform_simulation(P, N, pex, pim, t_max, classes=classes, pex2=pex2, pim2=pim2,inds=inds)
   tc_nacc <- make_naac_vector(tcs, t_max)
   post_sim_tab[k, 2] <- sum(abs(tc_nacc - data_nacc_vec))
   tc_naac_list[[k]] <- tc_nacc
}

pdf(file=paste("figures/", dataset, "_compare_mods.pdf", sep=""), height=4, width=5)
hist(post_sim_tab[,2], xlim=c(10,250), ylim=c(0,400), breaks=12, col=rgb(0,0,1,1/4),xlab="sAAC",main="")
par(new=TRUE)
hist(post_sim_tab[,1], xlim=c(10,250), ylim=c(0,400), breaks=12, col=rgb(1,0,0,1/4), xlab="sAAC",main="")
dev.off()

i_tc_best    <- which.min(post_sim_tab[,2])
tc_best_naac <- tc_naac_list[[ i_tc_best ]] 
pdf(file=paste("figures/", dataset, "_naac_tc.pdf", sep=""), height=5, width=5)
plot(tc_best_naac,xlim=c(3000,5000), xlab="step", ylab="arrivals (normalized)")
points(data_nacc_vec,col="blue")
dev.off()

save.image(file=paste("figures/", dataset, "_analyses.v2.Rd", sep="") )



